import junit.framework.*;
import java.util.NoSuchElementException;

/**
 * Unit tests for BasicLinkedList.<br>
 * CSE 373 demonstration program2006
 * @author Hal Perkins
 * @version 1/5/06, 1/10/07
 */
public class BasicLinkedListTest extends TestCase {
  
  // Data and setup common to several tests
  private BasicLinkedList testList;
  
  /** common setup for several tests */
  public void setUp() {
    testList = new BasicLinkedList();
    testList.add("abc");
    testList.add("pqr");
    testList.add("xyz");
  }
  
  // return true if list a contains strings in s
  private void checkContents(BasicLinkedList a, Object[] s) {
    assertEquals(s.length, a.size());
    for (int k = 0; k < s.length; k++) {
      assertTrue(s[k].equals(a.get(k)));
    }
  }
  
  /** test constructor and size, isEmpty on empty list */
  public void testConstructor() {
    BasicLinkedList a = new BasicLinkedList();
    assertEquals(0, a.size());
    assertTrue(a.isEmpty());
  }
  
  /** test add on empty list */
  public void testAddGetSimple() {
    BasicLinkedList a = new BasicLinkedList();
    a.add("one");
    assertEquals(1, a.size());
    assertFalse(a.isEmpty());
    assertTrue(a.get(0).equals("one"));
  }
  
  /** test multiple adds and gets */
  public void testAddGet() {
    String[] expected = {"abc", "pqr", "xyz"};
    checkContents(testList, expected);
  }
  
  /** test addFirst on empty and non-empty lists */
  public void testAddFirst() {
    BasicLinkedList a = new BasicLinkedList();
    a.addFirst("xyzzy");
    String[] ex1 = {"xyzzy"};
    checkContents(a, ex1);
    
    testList.addFirst("xyzzy");
    String[] ex2 = {"xyzzy", "abc", "pqr", "xyz"};
    checkContents(testList, ex2);
  }
  
  /** test indexOf on non-empty list */
  public void testIndexOf() {
    assertEquals(0, testList.indexOf("abc"));
    assertEquals(1, testList.indexOf("pqr"));
    assertEquals(2, testList.indexOf("xyz"));
    assertEquals(-1, testList.indexOf("xyzzy"));
  }
  
  /** test contains on non-empty list */
  public void testContains() {
    assertTrue(testList.contains("abc"));
    assertTrue(testList.contains("pqr"));
    assertTrue(testList.contains("xyz"));
    assertFalse(testList.contains("xyzzy"));
  }
  
  /** test clear */
  public void testClear() {
    testList.clear();
    assertEquals(0, testList.size());
    assertTrue(testList.isEmpty());
  }
  
  /** verify that get throws exceptions as expected */
  public void testGetExceptions() {
    BasicLinkedList empty = new BasicLinkedList();
    try {
      Object o = empty.get(0);
      fail("IndexOutOfBoundsException not thrown");
    } catch (IndexOutOfBoundsException e) {
      // success
    }
    try {
      Object o = testList.get(-1);
      fail("IndexOutOfBoundsException not thrown");
    } catch (IndexOutOfBoundsException e) {
      // success
    }
    try {
      Object o = testList.get(testList.size());
      fail("IndexOutOfBoundsException not thrown");
    } catch (IndexOutOfBoundsException e) {
      // success
    }
  }
  
  /** test set with and without exceptions */
  public void testSet() {
    // check empty list
    BasicLinkedList empty = new BasicLinkedList();
    try {
      Object o = empty.set(0, "whoops");
      fail("IndexOutOfBoundsException not thrown");
    } catch (IndexOutOfBoundsException e) {
      // success
    }
    
    // check beginning, middle, end
    assertEquals("abc", testList.set(0, "zero"));
    String[] ex0 = {"zero", "pqr", "xyz"};
    checkContents(testList, ex0);
    
    assertEquals("pqr", testList.set(1, "one"));
    String[] ex1 = {"zero", "one", "xyz"};
    checkContents(testList, ex1);
    
    assertEquals("xyz", testList.set(2, "two"));
    String[] ex2 = {"zero", "one", "two"};
    checkContents(testList, ex2);
    
    // check out of bounds index
    try {
      Object o = testList.set(-1, "whoops");
      fail("IndexOutOfBounds exception not thrown");
    } catch (IndexOutOfBoundsException e) {
      // success
    }
    try {
      Object o = testList.set(testList.size(), "whoops");
      fail("IndexOutOfBounds exception not thrown");
    } catch (IndexOutOfBoundsException e) {
      // success
    }
  }
  
  //###TODO  not tested: add at position, all flavors of remove
  
  /** test toString on empty and non-empty lists */
  public void testToString() {
    BasicLinkedList a = new BasicLinkedList();
    assertEquals("[]", a.toString());
    assertEquals("[abc,pqr,xyz]", testList.toString());
  }
  
  /** Test basic iterator operations hasNext() and next() 
   * for empty and non-empty lists */
  public void testIteratorBasics() {
    BasicLinkedList a = new BasicLinkedList();
    BasicListIterator at = a.iterator();
    assertFalse(at.hasNext());
    try {
      Object o = at.next();
      fail("NoSuchElementException not thown");
    } catch (NoSuchElementException e) {
      // success
    }
    
    BasicListIterator it = testList.iterator();
    assertTrue(it.hasNext());
    assertEquals("abc", it.next());
    assertTrue(it.hasNext());
    assertEquals("pqr", it.next());
    assertTrue(it.hasNext());
    assertEquals("xyz", it.next());
    assertFalse(it.hasNext());
    try {
      Object o = it.next();
      fail("NoSuchElementException not thown");
    } catch (NoSuchElementException e) {
      // success
    }
  }
  
}